/**
 * @file		RSPTexture_datatypes.h
 * @date		25/08/2022
 * @author		JackCarterSmith
 * @copyright	GPL-v3.0
 * @brief		RSP Texture workflow structures definitions
 *
 */

#ifndef RSPTEXTURELIB_DATATYPES_H_
#define RSPTEXTURELIB_DATATYPES_H_

#ifdef __cplusplus
extern "C" {
#endif

///////////////////////////////////////////////////////////////////////////////
//                         Configuration structure
///////////////////////////////////////////////////////////////////////////////

typedef union u_rsptexture_parameters {
	struct {
		unsigned char verbose_mode:1;	//!< Output simple details about ID and other "light" things.
		unsigned char debug_mode:1;		//!< Output all values of faces, indices and vertices and others "heavy" things.
		unsigned char god_mode:1;		//!< Dev only. Output experimental values.

		unsigned char reserved0:5;		//!< For future use.
	};
	unsigned char raw;	//!< Raw options access for bit-masking or memory copy/compare.
} RSPTEXTURE_PARAMETERS ;


////////////////////////////////////////////////////////////////////////////////
//                         Lib's structure definitions
////////////////////////////////////////////////////////////////////////////////

#ifndef MEMFILE_DEF
typedef char* MEMFILE;
#define MEMFILE_DEF
#endif

#ifndef T_R8G8B8_DEF
typedef struct r8g8b8 { unsigned char r,g,b; } T_R8G8B8;
#define T_R8G8B8_DEF
#endif

#ifndef T_R8G8B8A8_DEF
typedef struct r8g8b8a8 { unsigned char r,g,b,a; } T_R8G8B8A8;
#define T_R8G8B8A8_DEF
#endif

typedef enum e_mat_type {
	RSPTEX_MAT_TYPE_TEXTURED = 1,
	RSPTEX_MAT_TYPE_SOLID = 2
} RSPTEX_MAT_TYPE;

typedef unsigned char T_SAMPLE;

typedef struct rsptexture_texture {
	unsigned char name[17];			// 16 + 1 string ending \0
	unsigned short width;
	unsigned short height;

	unsigned char type;
	unsigned char sample_bits;
	unsigned int palette_entries;
	T_R8G8B8A8 alpha_color;

	T_R8G8B8A8* pixels;
} T_RSPTEXTURE_TEXTURE;

typedef struct rsptexture_material {
	unsigned short id;
	unsigned char name[17];			// 16 + 1 string ending \0
	RSPTEX_MAT_TYPE type;

	float opacity;	//TODO: temporary, need to be defined
	float ambient;	//TODO: temporary, need to be defined

	T_RSPTEXTURE_TEXTURE* texture;
} T_RSPTEXTURE_MATERIAL;

typedef struct rsptexture_hmt {
	unsigned int materials_count;
	unsigned int textures_count;
	unsigned int texture_offset;	//TODO: useless?

	T_RSPTEXTURE_MATERIAL* materials;
	T_RSPTEXTURE_TEXTURE* textures;
} T_RSPTEXTURE_HMT;

#ifdef __cplusplus
}
#endif

#endif /* RSPTEXTURELIB_DATATYPES_H_ */
