/**
 * @file		RSPTexture.c
 * @date		14/09/2022
 * @author		JackCarterSmith
 * @copyright	GPL-v3.0
 * @brief		HMT textures datas parser and export to PNG format.
 *
 */

#if defined(RSPTEXTURE_DLL)
#	define RSPTEXTURE_DLLBUILD
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "config.h"
#include "RSPTexture_errordefs.h"
#include "RSPTexture_datatypes.h"
#include "hmt_parser.h"
#include "RSPTexture.h"


/*
 * Libs interface
 */
inline char* RSPTexture_getVersion( void ) {
	return PRG_VERSION;
}

/*
 * HMT parser
 */
T_RSPTEXTURE_HMT* RSPTexture_createHMT( void ) {
	return calloc(1, sizeof(T_RSPTEXTURE_HMT));
}

unsigned short RSPTexture_processHMTFile( T_RSPTEXTURE_HMT* hmtStruct, const char* const filePath,
		const RSPTEXTURE_PARAMETERS params ) {

	if ( hmtStruct == NULL || filePath == NULL ) return RSPLIB_ERROR_ARGS_NULL;

	return RSP_TextureLib_ParseHMTFile(filePath, hmtStruct, &params);
}

unsigned short RSPTexture_processHMTFileMemory( T_RSPTEXTURE_HMT* hmtStruct, const void* const memFilePtr,
		const long memFileSize, const RSPTEXTURE_PARAMETERS params ) {

	if ( hmtStruct == NULL || memFilePtr == NULL ) return RSPLIB_ERROR_ARGS_NULL;

	return RSP_TextureLib_ParseHMTMemFile((MEMFILE)memFilePtr, hmtStruct, &params);
}

/*
 * Material utilities
 */
T_RSPTEXTURE_MATERIAL* RSPTexture_getMaterialFromID( const T_RSPTEXTURE_HMT* pHmt, const unsigned short mat_id ) {
	if ( pHmt == NULL ) return NULL;
	if ( pHmt->materials == NULL ) return NULL;

	return &(pHmt->materials[mat_id]);
}

inline char* RSPTexture_getMaterialName( const T_RSPTEXTURE_MATERIAL* mat ) {
	if ( mat == NULL ) return NULL;

	return (char*)mat->name;
}

inline float RSPTexture_getMaterialOpacity( const T_RSPTEXTURE_MATERIAL* mat ) {
	if ( mat == NULL ) return 1.0;

	return mat->opacity;
}

inline float RSPTexture_getMaterialAmbient( const T_RSPTEXTURE_MATERIAL* mat ) {
	if ( mat == NULL ) return 1.0;

	return mat->ambient;
}

void RSPTexture_freeHMT( T_RSPTEXTURE_HMT* hmtStruct ) {
	unsigned int i;

	if (hmtStruct == NULL) return;

	if ((hmtStruct->textures_count > 0) && hmtStruct->textures) {
		for ( i = 0; i < hmtStruct->textures_count; i++ )
			if (hmtStruct->textures[i].pixels) free(hmtStruct->textures[i].pixels);
	}

	if (hmtStruct->materials) free(hmtStruct->materials);

	free(hmtStruct);
}
