/**
 * @file		hmt_parser.h
 * @date		26/08/2022
 * @author		JackCarterSmith
 * @copyright	GPL-v3.0
 * @brief		Process HMP file structure and extract its datas.
 *
 */

#include "RSPTexture_datatypes.h"


#ifndef RSPTEXTURELIB_HMT_PARSER_H_
#define RSPTEXTURELIB_HMT_PARSER_H_

/**
 * @brief 	Process HMT file stored in memory.
 * @details Parser will directly extract tiles count and information stored in
 * 			HMT file and store them in T_RSPTEXTURE_HMT structure.
 * @note 	Unmanaged mode
 *
 * @param[in]	pMemFile	Pointer to an in-memory HOB file location.
 * @param[out]	hmtStruct	Allocated empty T_RSPTEXTURE_HMT structure instance to
 * 							be filled with HMP datas.
 * @param[in]	pParams		Parser options. See RSPTEXTURE_PARAMETERS.
 *
 * @return Processing error code, RSPLIB_SUCCESS if no error.
 */
unsigned short RSP_TextureLib_ParseHMTMemFile(const MEMFILE pMemFile,
		T_RSPTEXTURE_HMT* hmtStruct, const RSPTEXTURE_PARAMETERS* pParams);

/**
 * @brief 	Process HMT file in file system.
 * @details HMP file is dumped in memory before parsing in order to enhance
 * 			performance during parser operation and	in optic to protect data
 * 			stored in the original file (read-only).
 * 			Parser will extract tiles count and information stored in HMT file
 * 			and store them in T_RSPTEXTURE_HMT structure.
 * @note 	Managed mode
 *
 * @param[in]	fileName	String value of file name/path.
 * @param[out]	hmtStruct	Allocated empty T_RSPTEXTURE_HMT structure instance to
 * 							be filled with HMP datas.
 * @param[in]	pParams		Parser options. See RSPTEXTURE_PARAMETERS.
 *
 * @return Processing error code, RSPLIB_SUCCESS if no error.
 */
unsigned short RSP_TextureLib_ParseHMTFile(const char* fileName,
		T_RSPTEXTURE_HMT* hmtStruct, const RSPTEXTURE_PARAMETERS* pParams);

#endif /* RSPTEXTURELIB_HMT_PARSER_H_ */
